/* FLUTPP.H - File containing the definition of comparison, arithmetic
   and auxiliary operators for Ereal-type numbers

   Eduardo T Ochiai revised 1999/03/03 ver 1.1

   Sorry, function names are all in portuguese; anyway I translated
   each name in the best way possible in the form of comments like :
       <portuguese-version>=<english version>
   For example,
       NUMERO=NUMBER.

   Please note that this is code for C++ Builder 1.0, it will NOT
   compile ok under C++ Builder 3.0 (I tried it). If you wish to make
   your own adjustments, go on !
   -------------------------------------------------------------
   Note on 1999/10/02 : Now this works in C++ Builder 4.0, I removed
   the _CLASSDEF line in "flutppc.h", and upgraded the .mak files to the
   C++ Builder 4.0 version.
   --------------------------------------------------------------

*/

#ifndef __FLUTPPC_H
#define __FLUTPPC_H

#include <vcl\sysutils.hpp>
#include <iostream.h>

#if !defined( __DEFS_H )
#include <_defs.h>
#endif

#define TRUE 1
#define FALSE 0

#define TAM_DOUBLE      25   // TAM=SIZE
#define DIGITOS_DOUBLE  15   // DIGITOS=DIGITS

#define ZERO  "0"
#define UM    "1"   // UM=ONE
#define EPS   "1E-60"
#define PI1_2 "1.5707963267948966192313216916397514420985"
#define PI    "3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117067982"
#define PI3_2 "4.7123889803846898576939650749192543262955"
#define PI_2  "6.283185307179586476925286766559005768394"
#define E     "2.7182818284590452353602874713526624977552"

#define max 64 //  max=max_tot+1
const max_int=3;     // always 3
const max_dec=60;  // number of decimal digits
const max_tot=63;  // total number of digits

class TMathError:public Exception {
  public:
  TMathError(const char *s):Exception(s) {};
};

class _CLASSTYPE Ereal {

public:
 // Operator =
 Ereal _FAR & operator=(const Ereal& x);

  // Copy constructor
 Ereal(const Ereal& x);

 // Auxiliary functions
 friend Ereal _FARFUNC subt_f(Ereal, Ereal);
 friend Ereal _FARFUNC mult_alg(Ereal, char);
 void desce_alg(char);
 void desce_2alg(char, char);
 int  tam_int();
 int  tam_dec();
 void desl_esq(int);
 void desl_dir(int);
 void zera();

 // Constructors
 Ereal();
 Ereal(char *number);

 // Destructor
 ~Ereal();

 // Conversion to double
 friend double _FARFUNC dbl(Ereal _FAR &);
 friend Ereal _FARFUNC re(double);

 // Conversion to int
 friend int _FARFUNC eint(Ereal _FAR &);

 // I/O functions
 friend void _FARFUNC le_numero(Ereal *x);
 friend char *texto(Ereal& _FAR, char *);

 // Operators and functions
 friend int _FARFUNC operator >(Ereal _FAR &, Ereal _FAR &);
 friend int _FARFUNC operator <(Ereal _FAR &, Ereal _FAR &);
 friend int _FARFUNC operator ==(Ereal _FAR &, Ereal _FAR &);
 friend int _FARFUNC operator !=(Ereal _FAR &, Ereal _FAR &);
 friend int _FARFUNC operator >=(Ereal _FAR &, Ereal _FAR &);
 friend int _FARFUNC operator <=(Ereal _FAR &, Ereal _FAR &);
 friend Ereal _FARFUNC operator +(Ereal _FAR &, Ereal _FAR &);
 friend Ereal _FARFUNC operator -(Ereal _FAR &, Ereal _FAR &);
 friend Ereal _FARFUNC operator *(Ereal _FAR &, Ereal _FAR &);
 friend Ereal _FARFUNC operator /(Ereal _FAR &, Ereal _FAR &);
 Ereal _FAR & operator+=(Ereal _FAR &);
 Ereal _FAR & operator-=(Ereal _FAR &);
 Ereal _FAR & operator*=(Ereal _FAR &);
 Ereal _FAR & operator/=(Ereal _FAR &);
 Ereal _FAR operator+();
 Ereal _FAR operator-();

 friend void _FARFUNC troca(Ereal _FAR &, Ereal _FAR &);
 friend Ereal _FARFUNC r(char *);
 friend Ereal _FARFUNC eleva(Ereal _FAR &, int);
 friend Ereal _FARFUNC fatorial(int&);
 friend Ereal _FARFUNC raiz(Ereal _FAR &);
 friend Ereal _FARFUNC absol(Ereal _FAR &);
 friend Ereal _FARFUNC frac(Ereal _FAR &);
 friend Ereal _FARFUNC inteiro(Ereal _FAR &);
 friend Ereal _FARFUNC exponencial(Ereal _FAR &);
 friend Ereal _FARFUNC seno(Ereal  &);
 friend Ereal _FARFUNC cosseno(Ereal _FAR &);

private:
 char *d;
 int e;
};

extern Ereal zero,um,eps,pi1_2,pi,pi3_2,pi_2,e;

extern int operator >(Ereal _FAR &, Ereal _FAR &);
extern int operator <(Ereal _FAR &, Ereal _FAR &);
extern int operator ==(Ereal _FAR &, Ereal _FAR &);
extern int operator !=(Ereal _FAR &, Ereal _FAR &);
extern int operator >=(Ereal _FAR &, Ereal _FAR &);
extern int operator <=(Ereal _FAR &, Ereal _FAR &);

extern void troca(Ereal _FAR &, Ereal _FAR &);
extern Ereal r(char*);

extern Ereal operator +(Ereal _FAR &, Ereal _FAR &);
extern Ereal operator -(Ereal _FAR &, Ereal _FAR &);
extern Ereal operator *(Ereal _FAR &, Ereal _FAR &);
extern Ereal operator /(Ereal _FAR &, Ereal _FAR &);
extern Ereal eleva(Ereal _FAR &, int n);
extern Ereal fatorial(int&);

extern Ereal subt_f(Ereal, Ereal);
extern Ereal mult_alg(Ereal, char);

extern Ereal raiz(Ereal _FAR &);
extern Ereal absol(Ereal _FAR &);
extern Ereal frac(Ereal _FAR &);
extern Ereal inteiro(Ereal _FAR &);

extern void mostra(Ereal _FAR &);
extern void le_numero(Ereal _FAR &);
extern char *texto(Ereal& _FAR, char *);

extern double dbl(Ereal _FAR &);
extern Ereal re(double);
extern int eint(Ereal _FAR &);

extern Ereal exponencial(Ereal _FAR &);
extern Ereal seno(Ereal _FAR &);
extern Ereal cosseno(Ereal _FAR &);
extern Ereal arctg(Ereal _FAR &);
#endif
